/*
 * Copyright (c) 2006 - 2007, Junya NAKATA <jnakata@kerrymansystems.jp>
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1] [arg2] [arg3] [arg4]"
	    "[arg5] [arg6] [arg7] [arg8]\n", p);
}

int
main(int argc, char *argv[])
{
    unsigned int	   *vi;
    float		   *vf;
    operand		   *src, *dst;

/*
 *	check if 8 command line arguments are given
 */
    if(argc != 9) {
	usage(argv[0]);
	return -1;
    }

/*
 *	allocate variables for arguments
 */
    if((src = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    if((dst = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }

/*
 *	convert source argument to packed value
 */
    vf = (float *)src->alligned;
    vf[3] = strtof(argv[1], NULL);
    vf[2] = strtof(argv[2], NULL);
    vf[1] = strtof(argv[3], NULL);
    vf[0] = strtof(argv[4], NULL);

/*
 *	convert destination argument to packed value
 */
    vf = (float *)dst->alligned;
    vf[3] = strtof(argv[5], NULL);
    vf[2] = strtof(argv[6], NULL);
    vf[1] = strtof(argv[7], NULL);
    vf[0] = strtof(argv[8], NULL);

/*
 *	show the contents of operands
 */
    vf = (float *)src->alligned;
    printf("CMPPS\n     source: (%12.4f, %12.4f, %12.4f, %12.4f)\n",
	    vf[3], vf[2], vf[1], vf[0]);
    vf = (float *)dst->alligned;
    vi = (unsigned int *)dst->alligned;
    printf("destination: (%12.4f, %12.4f, %12.4f, %12.4f)\n",
	    vf[3], vf[2], vf[1], vf[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps	(%0),	%%xmm0\n\t"
	"movaps	(%1),	%%xmm1\n\t"
	"cmpps	$00,	%%xmm0, %%xmm1\n\t"
	"movaps	%%xmm1,	(%1)\n"
	:
	:   "r"(src->alligned), "r"(dst->alligned)
	:   "%xmm0", "%xmm1"
    );

/*
 *	show the result
 */
    printf(" (EQ)result:      %08x,     %08x,     %08x,     %08x\n",
	    vi[3], vi[2], vi[1], vi[0]);


/*
 *	convert destination argument to packed value
 */
    vf[3] = strtof(argv[5], NULL);
    vf[2] = strtof(argv[6], NULL);
    vf[1] = strtof(argv[7], NULL);
    vf[0] = strtof(argv[8], NULL);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps	(%0),	%%xmm0\n\t"
	"movaps	(%1),	%%xmm1\n\t"
	"cmpps	$01,	%%xmm0, %%xmm1\n\t"
	"movaps	%%xmm1,	(%1)\n"
	:
	:   "r"(src->alligned), "r"(dst->alligned)
	:   "%xmm0", "%xmm1"
    );

/*
 *	show the result
 */
    printf(" (LT)result:      %08x,     %08x,     %08x,     %08x\n",
	    vi[3], vi[2], vi[1], vi[0]);

/*
 *	convert destination argument to packed value
 */
    vf[3] = strtof(argv[5], NULL);
    vf[2] = strtof(argv[6], NULL);
    vf[1] = strtof(argv[7], NULL);
    vf[0] = strtof(argv[8], NULL);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps	(%0),	%%xmm0\n\t"
	"movaps	(%1),	%%xmm1\n\t"
	"cmpps	$06,	%%xmm0, %%xmm1\n\t"
	"movaps	%%xmm1,	(%1)\n"
	:
	:   "r"(src->alligned), "r"(dst->alligned)
	:   "%xmm0", "%xmm1"
    );

/*
 *	show the result
 */
    printf("(NLE)result:      %08x,     %08x,     %08x,     %08x\n",
	    vi[3], vi[2], vi[1], vi[0]);

/*
 *	convert destination argument to packed value
 */
    vf[3] = strtof(argv[5], NULL);
    vf[2] = strtof(argv[6], NULL);
    vf[1] = strtof(argv[7], NULL);
    vf[0] = strtof(argv[8], NULL);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps	(%1),	%%xmm0\n\t"
	"cmpps	$00,	(%0),	%%xmm0\n\t"
	"movaps	%%xmm0,	(%1)\n"
	:
	:   "r"(src->alligned), "r"(dst->alligned)
	:   "%xmm0", "%xmm1"
    );

/*
 *	show the result
 */
    printf(" (EQ)result:      %08x,     %08x,     %08x,     %08x\n",
	    vi[3], vi[2], vi[1], vi[0]);

/*
 *	convert destination argument to packed value
 */
    vf[3] = strtof(argv[5], NULL);
    vf[2] = strtof(argv[6], NULL);
    vf[1] = strtof(argv[7], NULL);
    vf[0] = strtof(argv[8], NULL);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps	(%1),	%%xmm0\n\t"
	"cmpps	$01,	(%0),	%%xmm0\n\t"
	"movaps	%%xmm0,	(%1)\n"
	:
	:   "r"(src->alligned), "r"(dst->alligned)
	:   "%xmm0", "%xmm1"
    );

/*
 *	show the result
 */
    printf(" (LT)result:      %08x,     %08x,     %08x,     %08x\n",
	    vi[3], vi[2], vi[1], vi[0]);

/*
 *	convert destination argument to packed value
 */
    vf[3] = strtof(argv[5], NULL);
    vf[2] = strtof(argv[6], NULL);
    vf[1] = strtof(argv[7], NULL);
    vf[0] = strtof(argv[8], NULL);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps	(%1),	%%xmm0\n\t"
	"cmpps	$06,	(%0),	%%xmm0\n\t"
	"movaps	%%xmm0,	(%1)\n"
	:
	:   "r"(src->alligned), "r"(dst->alligned)
	:   "%xmm0", "%xmm1"
    );

/*
 *	show the result
 */
    printf("(NLE)result:      %08x,     %08x,     %08x,     %08x\n",
	    vi[3], vi[2], vi[1], vi[0]);

    free_operand(src);
    free_operand(dst);

    return 0;
}

