/*
 * Copyright (c) 2006 - 2007, Junya NAKATA <jnakata@kerrymansystems.jp>
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1] [arg2] [arg3] [arg4]\n", p);
}

int
main(int argc, char *argv[])
{
    double		   *v;
    operand		   *src, *dst;

/*
 *	check if 4 command line arguments are given
 */
    if(argc != 5) {
	usage(argv[0]);
	return -1;
    }

/*
 *	allocate variables for arguments
 */
    if((src = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    if((dst = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }

/*
 *	convert source argument to packed value
 */
    v = (double *)src->alligned;
    v[1] = strtod(argv[1], NULL);
    v[0] = strtod(argv[2], NULL);

/*
 *	convert destination argument to packed value
 */
    v = (double *)dst->alligned;
    v[1] = strtod(argv[3], NULL);
    v[0] = strtod(argv[4], NULL);

/*
 *	show the contents of operands
 */
    v = (double *)src->alligned;
    printf("CMPSD\n     source: (%12.4f, %12.4f)\n", v[1], v[0]);
    v = (double *)dst->alligned;
    printf("destination: (%12.4f, %12.4f)\n", v[1], v[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps	(%0),	%%xmm0\n\t"
	"movaps	(%1),	%%xmm1\n\t"
	"cmpsd	$0x00,	%%xmm0, %%xmm1\n\t"
	"movaps	%%xmm1,	(%1)\n"
	:
	:   "r"(src->alligned), "r"(dst->alligned)
	:   "%xmm0", "%xmm1"
    );

/*
 *	show the result
 */
    printf(" (EQ)result: (%12.4f, %12.4f)\n", v[1], v[0]);

/*
 *	convert destination argument to packed value
 */
    v[1] = strtod(argv[3], NULL);
    v[0] = strtod(argv[4], NULL);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps	(%0),	%%xmm0\n\t"
	"movaps	(%1),	%%xmm1\n\t"
	"cmpsd	$0x01,	%%xmm0, %%xmm1\n\t"
	"movaps	%%xmm1,	(%1)\n"
	:
	:   "r"(src->alligned), "r"(dst->alligned)
	:   "%xmm0", "%xmm1"
    );

/*
 *	show the result
 */
    printf(" (LT)result: (%12.4f, %12.4f)\n", v[1], v[0]);

/*
 *	convert destination argument to packed value
 */
    v[1] = strtod(argv[3], NULL);
    v[0] = strtod(argv[4], NULL);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps	(%0),	%%xmm0\n\t"
	"movaps	(%1),	%%xmm1\n\t"
	"cmpsd	$0x06,	%%xmm0, %%xmm1\n\t"
	"movaps	%%xmm1,	(%1)\n"
	:
	:   "r"(src->alligned), "r"(dst->alligned)
	:   "%xmm0", "%xmm1"
    );

/*
 *	show the result
 */
    printf("(NLE)result: (%12.4f, %12.4f)\n", v[1], v[0]);

/*
 *	convert destination argument to packed value
 */
    v[1] = strtod(argv[3], NULL);
    v[0] = strtod(argv[4], NULL);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps	(%1),	%%xmm0\n\t"
	"cmpsd	$0x00,	(%0),	%%xmm0\n\t"
	"movaps	%%xmm0,	(%1)\n"
	:
	:   "r"(src->alligned), "r"(dst->alligned)
	:   "%xmm0"
    );

/*
 *	show the result
 */
    printf(" (EQ)result: (%12.4f, %12.4f)\n", v[1], v[0]);

/*
 *	convert destination argument to packed value
 */
    v[1] = strtod(argv[3], NULL);
    v[0] = strtod(argv[4], NULL);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps	(%1),	%%xmm0\n\t"
	"cmpsd	$0x01,	(%0),	%%xmm0\n\t"
	"movaps	%%xmm0,	(%1)\n"
	:
	:   "r"(src->alligned), "r"(dst->alligned)
	:   "%xmm0"
    );

/*
 *	show the result
 */
    printf(" (LT)result: (%12.4f, %12.4f)\n", v[1], v[0]);

/*
 *	convert destination argument to packed value
 */
    v[1] = strtod(argv[3], NULL);
    v[0] = strtod(argv[4], NULL);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps	(%1),	%%xmm0\n\t"
	"cmpsd	$0x06,	(%0),	%%xmm0\n\t"
	"movaps	%%xmm0,	(%1)\n"
	:
	:   "r"(src->alligned), "r"(dst->alligned)
	:   "%xmm0"
    );

/*
 *	show the result
 */
    printf("(NLE)result: (%12.4f, %12.4f)\n", v[1], v[0]);

    free_operand(src);
    free_operand(dst);

    return 0;
}

