/*
 * Copyright (c) 2006 - 2007, Junya NAKATA <jnakata@kerrymansystems.jp>
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1] [arg2]\n", p);
}

int
main(int argc, char *argv[])
{
    double		   *v;
    operand		   *src, *dst;

/*
 *	check if 2 command line arguments are given
 */
    if(argc != 3) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    if((dst = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    v = (double *)src->alligned;
    v[1] = strtod(argv[1], NULL);
    v[0] = strtod(argv[2], NULL);

/*
 *	show the contents of operands
 */
    printf("MOVAPD\n     source: (%12.4f, %12.4f)\n", v[1], v[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movapd	(%0),	%%xmm0\n\t"
	"movapd	%%xmm0,	%%xmm1\n\t"
	"movapd	%%xmm1,	%%xmm0\n\t"
	"movapd	%%xmm1,	(%1)\n"
	:
	:   "r"(src->alligned), "r"(dst->alligned)
	:   "%xmm0", "%xmm1"
    );

/*
 *	show the result
 */
    v = (double *)dst->alligned;
    printf("     result: (%12.4f, %12.4f)\n", v[1], v[0]);

    free_operand(src);
    free_operand(dst);

    return 0;
}

