/*
 * Copyright (c) 2006 - 2007, Junya NAKATA <jnakata@kerrymansystems.jp>
 * All rights reserved.
 */

#include <stdio.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1]\n", p);
}

int
main(int argc, char *argv[])
{
    operand		   *src, *dst0, *dst1;

/*
 *	check if 1 command line arguments are given
 */
    if(argc != 2) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src = hex2operand64(argv[1], 8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((dst0 = alloc_operand64(8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    if((dst1 = alloc_operand64(8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }

/*
 *	show the contents of operands
 */
    printf("MOVD\n     source: %s\n", operand2str(src));

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"pxor	%%mm0,	%%mm0\n\t"
	"movd	(%0),	%%mm0\n\t"
	"movd	%%mm0,	(%1)\n\t"
	"movd	%%mm0,	%%eax\n\t"
	"movl	%%eax,	(%2)\n\t"
	"emms\n"
	:
	:   "r"(src->alligned), "r"(dst0->alligned), "r"(dst1->alligned)
	:   "%eax", "%mm0"
    );

/*
 *	show the result
 */
    printf("     result: %s\n", operand2str(dst0));
    printf("     result: %s\n", operand2str(dst1));

    free_operand(src);
    free_operand(dst0);
    free_operand(dst1);

    return 0;
}

