/*
 * Copyright (c) 2006 - 2007, Junya NAKATA <jnakata@kerrymansystems.jp>
 * All rights reserved.
 */

#include <stdio.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1]\n", p);
}

int
main(int argc, char *argv[])
{
    operand		   *src, *tmp, *dst;

/*
 *	check if 1 command line arguments are given
 */
    if(argc != 2) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src = hex2operand128(argv[1], 16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((tmp = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    if((dst = alloc_operand64(8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }

/*
 *	show the contents of operands
 */
    printf("MOVDQ2Q\n     source: %s\n", operand2str(src));

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movdqa	(%0),	%%xmm0\n\t"
	"movdq2q	%%xmm0,	%%mm0\n\t"
	"movdqa	%%xmm0,	(%1)\n\t"
	"movq	%%mm0,	(%2)\n\t"
	"emms\n"
	:
	:   "r"(src->alligned), "r"(tmp->alligned), "r"(dst->alligned)
	:   "%mm0", "%xmm0"
    );

/*
 *	show the result
 */
    printf("   temporal: %s\n", operand2str(tmp));
    printf("     result: %s\n", operand2str(dst));

    free_operand(src);
    free_operand(dst);

    return 0;
}

