/*
 * Copyright (c) 2006 - 2007, Junya NAKATA <jnakata@kerrymansystems.jp>
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1]\n", p);
}

int
main(int argc, char *argv[])
{
    double		   *v;
    operand		   *src, *tmp, *dst;

/*
 *	check if 1 command line arguments are given
 */
    if(argc != 2) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src = alloc_operand64(8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    if((tmp = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    if((dst = alloc_operand64(8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    v = (double *)src->alligned;
    *v = strtod(argv[1], NULL);

/*
 *	show the contents of operands
 */
    printf("MOVLPD\n     source: %12.4f\n", *v);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"xorpd	%%xmm0,	%%xmm0\n\t"
	"movlpd	(%0),	%%xmm0\n\t"
	"movapd	%%xmm0,	(%1)\n\t"
	"movlpd	%%xmm0,	(%2)\n"
	:
	:   "r"(src->alligned), "r"(tmp->alligned), "r"(dst->alligned)
	:   "%xmm0"
    );

/*
 *	show the result
 */
    v = (double *)tmp->alligned;
    printf("     result: (%12.4f, %12.4f)\n", v[1], v[0]);
    v = (double *)dst->alligned;
    printf("     result: %12.4f\n", *v);

    free_operand(src);
    free_operand(dst);

    return 0;
}

