/*
 * Copyright (c) 2006 - 2007, Junya NAKATA <jnakata@kerrymansystems.jp>
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1] [arg2]\n", p);
}

int
main(int argc, char *argv[])
{
    float		   *v;
    operand		   *src, *tmp, *dst;

/*
 *	check if 2 command line arguments are given
 */
    if(argc != 3) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src = alloc_operand64(8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    if((tmp = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    if((dst = alloc_operand64(8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    v = (float *)src->alligned;
    v[1] = strtof(argv[1], NULL);
    v[0] = strtof(argv[2], NULL);

/*
 *	show the contents of operands
 */
    printf("MOVLPS\n     source: (%12.4f, %12.4f)\n", v[1], v[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"xorps	%%xmm0,	%%xmm0\n\t"
	"movlps	(%0),	%%xmm0\n\t"
	"movaps	%%xmm0,	(%1)\n"
	"movlps	%%xmm0,	(%2)\n"
	:
	:   "r"(src->alligned), "r"(tmp->alligned), "r"(dst->alligned)
	:   "%xmm0"
    );

/*
 *	show the result
 */
    v = (float *)tmp->alligned;
    printf("   temporal: (%12.4f, %12.4f, %12.4f, %12.4f)\n",
	    v[3], v[2], v[1], v[0]);

    v = (float *)dst->alligned;
    printf("     result: (%12.4f, %12.4f)\n", v[1], v[0]);

    free_operand(src);
    free_operand(tmp);
    free_operand(dst);

    return 0;
}

