/*
 * Copyright (c) 2006 - 2007, Junya NAKATA <jnakata@kerrymansystems.jp>
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1] [arg2] [arg3] [arg4]\n", p);
}

int
main(int argc, char *argv[])
{
    int			    dst;
    float		   *v;
    operand		   *src;

/*
 *	check if 4 command line arguments are given
 */
    if(argc != 5) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    v = (float *)src->alligned;
    v[3] = strtof(argv[1], NULL);
    v[2] = strtof(argv[2], NULL);
    v[1] = strtof(argv[3], NULL);
    v[0] = strtof(argv[4], NULL);

/*
 *	show the contents of operands
 */
    printf("MOVMSKPS\n     source: (%12.4f, %12.4f, %12.4f, %12.4f)\n",
	    v[3], v[2], v[1], v[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps	(%0),	%%xmm0\n\t"
	"movmskps   %%xmm0, %%eax\n\t"
	"movl	%%eax, (%1)\n"
	:
	:   "r"(src->alligned), "r"(&dst)
	:   "%eax", "%xmm0"
    );

/*
 *	show the result
 */
    printf("     result: %04x\n", dst);

    free_operand(src);

    return 0;
}

