/*
 * Copyright (c) 2006 - 2007, Junya NAKATA <jnakata@kerrymansystems.jp>
 * All rights reserved.
 */

#include <stdio.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1] [arg2]\n", p);
}

int
main(int argc, char *argv[])
{
    operand		   *src;
    unsigned int	    dst;

/*
 *	check if 1 command line arguments are given
 */
    if(argc != 2) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src = hex2operand64(argv[1], 8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }

/*
 *	show the contents of operands
 */
    printf("PMOVMSKB\n     source: %s\n", operand2str(src));

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movq	(%0),	%%mm0\n\t"
	"pmovmskb	%%mm0,	%%eax\n\t"
	"movl	%%eax,	(%1)\n\t"
	"emms\n"
	:
	:   "r"(src->alligned), "r"(&dst)
	:   "%eax", "%mm0"
    );

/*
 *	show the result
 */
    printf("     result: %04x\n", dst);

    free_operand(src);

    return 0;
}

