/*
 * Copyright (c) 2006 - 2007, Junya NAKATA <jnakata@kerrymansystems.jp>
 * All rights reserved.
 */

#include <stdio.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1] [arg2]\n", p);
}

int
main(int argc, char *argv[])
{
    operand		   *src0, *dst0, *src1, *dst1;

/*
 *	check if 2 command line arguments are given
 */
    if(argc != 3) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src0 = hex2operand128(argv[1], 16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((dst0 = hex2operand128(argv[2], 16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg2(%s): invalid value.\n", argv[2]);
	return -1;
    }
    if((src1 = hex2operand128(argv[1], 16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((dst1 = hex2operand128(argv[2], 16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg2(%s): invalid value.\n", argv[2]);
	return -1;
    }

/*
 *	show the contents of operands
 */
    printf("PSUBUSW\n     source: %s\ndestination: %s\n",
	    operand2str(src0), operand2str(dst0));

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movdqa	(%0),	%%xmm0\n\t"
	"movdqa	(%1),	%%xmm1\n\t"
	"psubusw	%%xmm0,	%%xmm1\n\t"
	"movdqa	%%xmm1,	(%1)\n"
	:
	:   "r"(src0->alligned), "r"(dst0->alligned)
	:   "%xmm0", "%xmm1"
    );

/*
 *	show the result
 */
    printf("     result: %s\n", operand2str(dst0));

    free_operand(src0);
    free_operand(dst0);

/*
 *	show the contents of operands */
    printf("     source: %s\ndestination: %s\n",
	    operand2str(src1), operand2str(dst1));

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movdqa	(%1),	%%xmm0\n\t"
	"psubusw	(%0),	%%xmm0\n\t"
	"movdqa	%%xmm0,	(%1)\n"
	:
	:   "r"(src1->alligned), "r"(dst1->alligned)
	:   "%xmm0"
    );

/*
 *	show the result
 */
    printf("     result: %s\n", operand2str(dst1));

    free_operand(src1);
    free_operand(dst1);

    return 0;
}

